@@ -7,14 +7,15 @@
 char* word_wrap(char* str,int LINE_LENGTH);
 char* word_wrap_indented(char* str,int LINE_LENGTH,int nspace);
 extern void    ToUpper(char* s);
-extern void    QuickSortD(F32PTR arr,I32PTR  index,I32 low,I32 high);
-extern void    QuickSortA(F32PTR arr,I32PTR  index,I32 low,I32 high);
+extern void    f32_QuickSortD(F32PTR arr,I32PTR  index,I32 low,I32 high);
+extern void    f32_QuickSortA(F32PTR arr,I32PTR  index,I32 low,I32 high);
 extern I32	   strcicmp(char const * _restrict a,char const * _restrict b);
 extern I32     strcicmp_nfirst(char const* _restrict a,char const* _restrict b,int nfirst);
 extern I32     strmatch(char const* _restrict full,char const* _restrict part);
 extern F32     DeterminePeriod(F32PTR Y,I32 N);
 extern I32     FindChangepoint(F32PTR prob,F32PTR mem,F32 threshold,I32PTR cpt,F32PTR cptCI,I32 N,I32 minSepDist,I32 maxCptNumber);
-extern void SetupRoutines_ByCPU(Bool quiet);
+extern I32     FindChangepoint_LeftRightMargins(F32PTR prob,F32PTR mem,F32 threshold,I32PTR cpt,F32PTR cptCI,I32 N,I32 minSepDist,I32 maxCptNumber,I32 leftMargin,I32 rightMargin);
+extern void SetupRoutines_AutoByCPU(Bool quiet);
 extern void SetupRoutines_UserChoice(int avxOption);
 #define    CopyInsertNewElement(SRC,NSRC,DST,newPos,newValue,T)                                     \
                          memcpy(DST,SRC,(newPos-1)*sizeof(T));        \
