@@ -6,6 +6,9 @@
 #include "abc_common.h"
 #include "abc_date.h"
 #if P_INTERFACE==1  
+int  JDN_to_DateNum(int jdn) {
+    return jdn - JulianDayNum_from_civil_ag1(1,1,1);
+}
 PyObject* currentModule;
 PyObject* classOutput=NULL;
 PyObject* setClassObjects(PyObject* self,PyObject* args){
@@ -106,9 +109,12 @@ int IsChar(void* ptr) {
     if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_STRING) {
         return 1;
     }
+    if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_UNICODE) {
+        return 1;
+    }
     PyGetItem pyGetItem=NULL;
-    if (PyList_Check(ptr))           pyGetItem=PyList_GetItem;
-    else if (PyTuple_Check(ptr))     pyGetItem=PyDict_GetItem;
+    if      (PyList_Check(ptr))        pyGetItem=PyList_GetItem;
+    else if (PyTuple_Check(ptr))       pyGetItem=PyDict_GetItem;
     if(pyGetItem) {      
         int sz=(int) PyObject_Size(ptr);
         int ok=1;
@@ -120,6 +126,21 @@ int IsChar(void* ptr) {
         }
         if (ok) { return 1; }
     }
+    if (PyArray_Check(ptr) && (PyArray_TYPE(ptr)==NPY_STRING||PyArray_TYPE(ptr)==NPY_UNICODE)  ){
+        return 1;
+    }
+    if (PyArray_Check(ptr)  && PyArray_TYPE(ptr)==NPY_OBJECT) {
+        void** base=(void**) PyArray_DATA(ptr);
+        int sz=(int)    PyArray_Size(ptr);
+        int ok=1;
+        for (int i=0; i < sz;++i) {
+            if (!PyUnicode_Check(base[i])) {
+                ok=0;
+                break;
+            }
+        }
+        if (ok) { return 1; }
+    }
     return 0;
 }
 int IsStruct(void* ptr) {
@@ -128,10 +149,16 @@ int IsStruct(void* ptr) {
         return 1L;
     }
     Pob dict=PyGetDict(ptr);
-    if (dict) return 1;
+    if (dict) {
+        if (PyUnicode_Check(ptr)) {
+            return 0;
+        }  else {
+            return 1;
+        }        
+    }
     return 0L;
 }
-int IsEmpty(void* ptr) {   return (ptr==Py_None||GetNumberOfElements(ptr)==0);}
+int IsEmpty( void* ptr) {  return (ptr==Py_None||GetNumberOfElements(ptr)==0);}
 int IsSingle(void* ptr) {
     if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_FLOAT32) {
         return 1L;        
@@ -159,6 +186,18 @@ int IsDouble(void* ptr) {
         }
         if (ok) { return 1; }
     }
+    if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_OBJECT) {
+        void** base=(void**)PyArray_DATA(ptr);
+        int sz=(int)PyArray_Size(ptr);
+        int ok=1;
+        for (int i=0; i < sz;++i) {
+            if (!PyFloat_Check(base[i])) {
+                ok=0;
+                break;
+            }
+        }
+        if (ok) { return 1; }
+    }
     return 0;
 }
 int IsInt16(void* ptr) {   
@@ -175,7 +214,7 @@ int IsInt32(void* ptr) {
         return 1L;
     }
     PyGetItem pyGetItem=NULL;
-    if (PyList_Check(ptr))           pyGetItem=PyList_GetItem;
+    if      (PyList_Check(ptr))      pyGetItem=PyList_GetItem;
     else if (PyTuple_Check(ptr))     pyGetItem=PyDict_GetItem;
     if (pyGetItem) {
         int sz=PyObject_Size(ptr);
@@ -188,6 +227,18 @@ int IsInt32(void* ptr) {
         }
         if (ok) { return 1; }
     }
+    if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_OBJECT) {
+        void** base=(void**)PyArray_DATA(ptr);
+        int sz=(int)PyArray_Size(ptr);
+        int ok=1;
+        for (int i=0; i < sz;++i) {
+            if (!PyLong_Check(base[i])) {
+                ok=0;
+                break;
+            }
+        }
+        if (ok) { return 1; }
+    }
     return 0;
 }
 int IsInt64(void* ptr) {
@@ -204,7 +255,7 @@ int IsLogical(void* ptr) {
         return 1L;
     }
     PyGetItem pyGetItem=NULL;
-    if (PyList_Check(ptr))           pyGetItem=PyList_GetItem;
+    if      (PyList_Check(ptr))      pyGetItem=PyList_GetItem;
     else if (PyTuple_Check(ptr))     pyGetItem=PyDict_GetItem;
     if (pyGetItem) {
         int sz=PyObject_Size(ptr);
@@ -217,6 +268,18 @@ int IsLogical(void* ptr) {
         }
         if (ok) { return 1; }
     }
+    if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_OBJECT) {
+        void** base=(void**)PyArray_DATA(ptr);
+        int sz=(int)PyArray_Size(ptr);
+        int ok=1;
+        for (int i=0; i < sz;++i) {
+            if (!PyBool_Check(base[i])) {
+                ok=0;
+                break;
+            }
+        }
+        if (ok) { return 1; }
+    }
     return 0;
 }
 int IsNumeric(void* ptr) {
@@ -304,44 +367,66 @@ I32   GetConsoleWidth() {
 }
 I32 GetCharArray(void* ptr,char* dst,int n) {
     dst[0]=0;
-    if (!IsChar(ptr)) return 0;
+    if (ptr==NULL||!IsChar(ptr)) return 0;
     if (PyUnicode_Check(ptr)) {
         Py_ssize_t  len;
         char* str=PyUnicode_AsUTF8AndSize(ptr,&len);
         strncpy(dst,str,n);
         return len;
     }
-    if (PyArray_Check(ptr)) {
-        r_printf("Not usre how to read strings out of a numpy str array!\n");
-        *dst=0;        
-    }
-    dst[n - 1]=0;
-    return 0;
+    int idx=0;
+    return  GetCharVecElem(ptr,idx,dst,n);
 }
 I32 GetCharVecElem(void* ptr,int idx,char* dst,int n) {
-    dst[0]=0;
-    if (!IsChar(ptr)) return 0;
-    Pob tmpItem=NULL;
-    if (PyUnicode_Check(ptr) && n==0) {
+    Py_ssize_t len=0; 
+    PyObject* tmpItem=NULL;
+    if (PyUnicode_Check(ptr) && idx==0) {
         tmpItem=ptr;
-    }
-    if (PyList_Check(ptr)) {
+    } else   if (PyList_Check(ptr)) {
         tmpItem=PyList_GetItem(ptr,idx);
-    }
-    if (PyTuple_Check(ptr)) {
+    } else  if (PyTuple_Check(ptr)) {
         tmpItem=PyTuple_GetItem(ptr,idx);
     }
-    if (PyUnicode_Check(tmpItem)) {
-        Py_ssize_t  len;
+    if (tmpItem && PyUnicode_Check(tmpItem)) {   
         char* str=PyUnicode_AsUTF8AndSize(tmpItem,&len);
-        strncpy(dst,str,n);
+        len=min(len,n - 1);
+        memcpy(dst,str,len);
+        dst[len]=0;
         return len;
     }
-    if (PyArray_Check(ptr)) {
-        r_printf("Not usre how to read strings out of a numpy str array!\n");
-        *dst=0;
+    if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_STRING) {
+        char* base=PyArray_DATA(ptr);
+        int   elsize=PyArray_ITEMSIZE(ptr);
+        char* str=base+elsize * idx;
+        len=min(elsize,n-1);
+        memcpy(dst,str,len);
+        dst[len]=0;
+        return len;
     }
-    dst[n - 1]=0;
+    if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_UNICODE) {
+        char* base=PyArray_DATA(ptr);
+        int   elsize=PyArray_ITEMSIZE(ptr);
+        char* str=base+elsize * idx;
+        len=min(elsize/4,n - 1);  
+        for (int i=0; i < len; i++) {
+            dst[i]=str[i * 4];
+        }
+        dst[len]=0;
+        return len;
+    }
+    if (PyArray_Check(ptr) && PyArray_TYPE(ptr)==NPY_OBJECT) {
+        void** base=PyArray_DATA(ptr);
+        int    elsize=PyArray_ITEMSIZE(ptr);
+        void* tmpItem=base[idx];
+        if (tmpItem && PyUnicode_Check(tmpItem)) {
+            char* str=PyUnicode_AsUTF8AndSize(tmpItem,&len);
+            len=min(len,n - 1);
+            memcpy(dst,str,len);
+            dst[len]=0;
+            return len;
+        }
+    }
+    dst[len]=0;
     return 0;
 }
 void* GetField123(const void* structVar,char* fname,int nPartial) {
@@ -441,7 +526,9 @@ F64   GetNumericElement(const void* Y,I32 idx0) {
     }
     return getNaN();
 }
-void* GetData(const void* ptr) { return PyArray_DATA(ptr);}
+void* GetData(const void* ptr) { 
+    return PyArray_DATA(ptr);
+}
 int  GetDim1(const void* ptr) {
     if (PyArray_Check(ptr)) {
         npy_intp* dims=PyArray_DIMS(ptr);
@@ -503,6 +590,9 @@ int  GetNumberOfElements(const void* ptr) {
     if (PyList_Check(ptr)||PyTuple_Check(ptr)) {
         return PyObject_Size(ptr);
     }
+    if (PyUnicode_Check(ptr) ) {
+        return 1;
+    }
     PyObject* dict=PyGetDict(ptr);
     if (dict) {
         return PyObject_Size(dict);
@@ -561,12 +651,15 @@ static void pySetAddField(void *obj,char *fname,void *value) {
     Py_XDECREF(value);  
 }
 void* CreateStructVar(FIELD_ITEM* fieldList,int nfields) {
-	int nfields_new=0;
+	int nfields_actual=0;
 	for (int i=0; i < nfields;++i) {
-		nfields_new++;
-		if (fieldList[i].name[0]==0) 	break;
+        nfields_actual++;
+        if (fieldList[i].name[0]==0) {
+            nfields_actual--;
+            break;
+        }
 	}
-	nfields=nfields_new; 
+	nfields=nfields_actual;
 	PyObject * _restrict out;
 	{
         out=pyCreateStructObject(); 
