@@ -1,7 +1,8 @@
 #include "abc_000_macro.h"
 #include "abc_000_warning.h"
+#include "abc_001_config.h"
 #include "abc_datatype.h"
- #if M_INTERFACE==1 && !defined(MSVC_COMPILER)
+ #if M_INTERFACE==1 &&  !defined(MSVC_COMPILER)
 	#include "inttypes.h"
 	#include "mex.h"
 	#if defined(WIN_OS)
@@ -12,7 +13,9 @@
 		extern Bool ioFlush(void)  asm("_Z7ioFlushv");
 	#endif
     void matlab_IOflush(void)	{
-		ioFlush();
+		#ifndef O_INTERFACE
+			ioFlush();
+		#endif
 	}
 #else
 static char achar UNUSED_DECORATOR='c';
