@@ -126,8 +126,7 @@ I32  GetInfoBandList(BEAST2_BASESEG* info,BEAST2_MODEL_PTR model,I32 Klastcol) {
 	I32 QUITFLAG=0;
 	for (int basisID=0; basisID < model->NUMBASIS; basisID++) {
 		BEAST2_BASIS_PTR b=model->b+basisID;
-		if (b->type !=OUTLIERID)
-		{
+		if (b->type !=OUTLIERID) {
 			I32 numSeg=b->nKnot+1;
 			for (int j=0; j < numSeg; j++) {
 				I32 Kbase=b->Kbase;
@@ -142,9 +141,8 @@ I32  GetInfoBandList(BEAST2_BASESEG* info,BEAST2_MODEL_PTR model,I32 Klastcol) {
 					QUITFLAG=1;	break;
 				}
 			}
-		}
-		else
-		{
+		} 
+		else {
 			I32 numSeg=b->nKnot;
 			for (int j=0; j < numSeg; j++) {
 				I32 Kbase=b->Kbase;
@@ -168,8 +166,7 @@ I32  GetInfoBandList_post(BEAST2_BASESEG* info,BEAST2_MODEL_PTR model,I32 Kstart
 	I32 numBands=0;
 	for (int basisID=0; basisID < model->NUMBASIS; basisID++) {
 		BEAST2_BASIS_PTR b=model->b+basisID;
-		if (b->type !=OUTLIERID)
-		{
+		if (b->type !=OUTLIERID) {
 			for (int j=0; j < b->nKnot+1; j++) {
 				I32  Kbase=b->Kbase;
 				if (Kstartcol <=(Kbase+b->ke[j])) {
@@ -181,8 +178,7 @@ I32  GetInfoBandList_post(BEAST2_BASESEG* info,BEAST2_MODEL_PTR model,I32 Kstart
 				}
 			}
 		}
-		else
-		{
+		else {
 			for (int j=0; j < b->nKnot; j++) {
 				I32  Kbase=b->Kbase;
 				if (Kstartcol <=(Kbase+b->ke[j])) {
