@@ -3,6 +3,7 @@
 #include "globalvars.h"
 VOID_PTR  GLOBAL_OPTIONS;
 VOID_PTR  GLOBAL_RESULT;
+VOID_PTR  GLOBAL_DATA0;
 pthread_t       *thread_id;
 pthread_mutex_t  mutex;
 pthread_cond_t   condVar;
@@ -11,5 +12,5 @@ volatile F32    REMAINING_TIME;
 volatile I32    NEXT_PIXEL_INDEX;
 volatile I32    NUM_OF_PROCESSED_PIXELS;
 volatile I32    NUM_OF_PROCESSED_GOOD_PIXELS;
-int  IS_CPU_INSTRUCTON_SET=0;
+char GLOBAL_QUIET_MODE=0;
 #include "abc_000_warning.h"
